import json
import os

from jinja2 import Template
import yaml


def _load_template(template_str):
    """ Loads jinja template object from string """
    return Template(template_str)


def _render_template(template, **params):
    """ Render a jinja template object """
    return template.render(**params)


def render_string(template_str, **params):
    """ Renders output from a jinja template string """
    template = _load_template(template_str)
    return _render_template(template, **params)


def _check_valid_path(path):
    """ Ensure that path exists and is a file """
    if not os.path.exists(path):
        raise ValueError(f'Path "{path}" does not exist.')
    if not os.path.isfile(path):
        raise TypeError(f'Path "{path}" is not a file.')


def _read_json(path):
    """ Read JSON file from path """
    _check_valid_path(path)
    with open(path) as _file:
        return json.load(_file)


def _read_yaml(path):
    """ Read YAML file from path """
    _check_valid_path(path)
    with open(path) as _file:
        return yaml.safe_load(_file)


def read_params(path):
    """ Read parameters from JSON or YAML file path """
    if path is None:
        return {}
    elif path.endswith('.json'):
        return _read_json(path)
    elif path.endswith(('.yml', '.yaml')):
        return _read_yaml(path)
    else:
        raise ValueError(f'Path "{path}" does not have a known extension.')


def write_output(output, path):
    """ Write output string to file at path """
    with open(path, 'w') as _file:
        _file.write(output)



import click

from kubetemp.core import read_params, render_string, write_output


@click.command()
@click.version_option()
@click.argument('TEMPLATE', type=click.File('r'))
@click.option(
    'param_path', '--file', '-f', type=click.Path(exists=True),
    help='Read parameters from JSON or YAML file'
)
@click.option(
    'param_list', '--param', '-p', nargs=2, multiple=True, type=str,
    help='Define parameters to render template'
)
@click.option(
    'output_path', '--output', '-o', type=click.Path(writable=True),
    help='Write rendered template to output file'
)
def cli(template, param_path, param_list, output_path):
    """ Renders Jinja template with parameters"""
    # Get template parameters
    params = {}
    file_params = read_params(param_path)
    params.update(file_params)
    params.update(dict(param_list))

    # Render template and create output
    rendered_output = render_string(template.read(), **params)
    if output_path is None:
        click.echo(rendered_output)
    else:
        write_output(rendered_output, output_path)



from setuptools import setup

setup(
    name='kubetemp',
    version='0.2.5',    # PyPI version automatically set by Travis CI
    author='Jared Vasquez',
    url='https://github.com/jgv7/kubetemp',
    long_description='https://github.com/jgv7/kubetemp',
    description=(
        'Tool for generating kubernetes (k8s) manifests using Jinja templates.'
    ),
    license='MIT',
    install_requires=[
        'Click>=7.0',
        'jinja2>=2.10',
        'PyYAML<=3.13,>=3.10',
    ],
    packages=['kubetemp'],
    python_requires=">=3.6",
    entry_points='''
        [console_scripts]
        kubetemp=kubetemp.cli:cli
    ''',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    zip_safe=False,
)
